@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009 Michael Lamothe
+ * Copyright (C) 2010 Michael Lamothe
  *
  * This file is part of Me TV
  *
@@ -21,34 +21,30 @@
 #ifndef __CHANNEL_H__
 #define __CHANNEL_H__
 
-#include <linux/dvb/frontend.h>
 #include "epg_events.h"
 #include "dvb_transponder.h"
-
-#define CHANNEL_FLAG_NONE		0
-#define CHANNEL_FLAG_DVB_T		0x01
-#define CHANNEL_FLAG_DVB_C		0x02
-#define CHANNEL_FLAG_DVB_S		0x04
-#define CHANNEL_FLAG_ATSC		0x08
+#include <linux/dvb/frontend.h>
 
 class Channel
 {
 public:
 	Channel();
 
-	guint			channel_id;
-	Glib::ustring	name;
-	guint			flags;
-	guint			sort_order;
-	Glib::ustring	mrl;
-	EpgEvents		epg_events;
-
-	// DVB Specific
+	guint				channel_id;
+	Glib::ustring		name;
+	guint				type;
+	guint				sort_order;
+	Glib::ustring		mrl;
+	EpgEvents			epg_events;
+	
 	guint				service_id;
 	Dvb::Transponder	transponder;
-	guint get_transponder_frequency();
+	
+	guint				get_transponder_frequency();
+	Glib::ustring		get_text();
 
-	Glib::ustring get_text();
+	bool operator==(const Channel& channel) const;
+	bool operator!=(const Channel& channel) const;
 };
 
 class ChannelArray : public std::vector<Channel>
