@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009 Michael Lamothe
+ * Copyright (C) 2010 Michael Lamothe
  *
  * This file is part of Me TV
  *
@@ -42,9 +42,11 @@ namespace Data
 		sqlite3*	connection;
 		gboolean	database_created;
 	public:
+		Connection();
 		Connection(const Glib::ustring& filename);
 		~Connection();
 
+		void open(const Glib::ustring& filename);
 		guint get_last_insert_rowid();
 		gboolean get_database_created() const { return database_created; }
 		void vacuum();
