@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009 Michael Lamothe
+ * Copyright (C) 2010 Michael Lamothe
  *
  * This file is part of Me TV
  *
@@ -92,7 +92,7 @@ void DevicesDialog::on_response(int response_id)
 				Application& application = get_application();
 				application.set_string_configuration_value("default_device", frontend->get_path());
 				application.device_manager.set_frontend(*frontend);
-				application.restart_stream();
+				application.channel_manager.select_display_channel();
 			}
 		}
 	}
