@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009 Michael Lamothe
+ * Copyright (C) 2010 Michael Lamothe
  *
  * This file is part of Me TV
  *
@@ -172,7 +172,7 @@ void EpgEvents::save(Data::Connection& connection, guint channel_id)
 
 	Glib::RecMutex::Lock lock(mutex);
 
-	g_debug("Saving %d EPG events", list.size());
+	g_debug("Saving %d EPG events", (int)list.size());
 	for (EpgEventList::iterator j = list.begin(); j != list.end(); j++)
 	{
 		EpgEvent& epg_event = *j;
