@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009 Michael Lamothe
+ * Copyright (C) 2010 Michael Lamothe
  *
  * This file is part of Me TV
  *
@@ -187,7 +187,7 @@ void GtkEpgWidget::update_table()
 			{
 				guint hour_time = start_time + (hour * 60 * 60);
 				Glib::ustring hour_time_text = get_local_time_text(hour_time, "%c");
-				Gtk::Button& button = attach_button(hour_time_text, hour * COLUMNS_PER_HOUR + 1, (hour+1) * COLUMNS_PER_HOUR + 1, 0, 1, Gtk::FILL | Gtk::EXPAND);
+				Gtk::Button& button = attach_button(hour_time_text, false, hour * COLUMNS_PER_HOUR + 1, (hour+1) * COLUMNS_PER_HOUR + 1, 0, 1, Gtk::FILL | Gtk::EXPAND);
 				button.set_sensitive(false);
 			}
 			row++;
@@ -231,16 +231,35 @@ void GtkEpgWidget::create_channel_row(const Channel& const_channel,
 		channel_text = Glib::ustring::compose("<i>%1.</i> ", channel_number) + channel_text;
 	}
 	
-	Gtk::ToggleButton& channel_button = attach_toggle_button( channel_text, 0, 1, table_row, table_row + 1);
+	gboolean record_channel = get_application().stream_manager.is_recording(const_channel);
+	
+	Gtk::ToggleButton& channel_button = attach_toggle_button(channel_text, record_channel, 0, 1, table_row, table_row + 1);
+
+	if (selected)
+	{
+		Gtk::HBox* hbox = dynamic_cast<Gtk::HBox*>(channel_button.get_child());
+		Gtk::Image* image = Gtk::manage(new Gtk::Image(Gtk::Stock::MEDIA_PLAY, Gtk::ICON_SIZE_BUTTON));
+		image->set_alignment(1, 0.5);
+		hbox->pack_start(*image, false, false);
+		image->show();
+	}
 	
 	channel_button.set_active(selected);
 	channel_button.signal_clicked().connect(
 		sigc::bind<guint>
 		(
-			sigc::mem_fun(*this, &GtkEpgWidget::on_button_channel_name_clicked),
+			sigc::mem_fun(*this, &GtkEpgWidget::on_button_channel_clicked),
 			channel.channel_id
 		)
 	);
+	channel_button.signal_button_press_event().connect(
+		sigc::bind<Channel>
+		(
+			sigc::mem_fun(*this, &GtkEpgWidget::on_button_channel_press_event),
+			channel
+		),
+	    false
+	);
 	
 	guint total_number_columns = 0;
 	guint end_time = start_time + epg_span_hours*60*60;
@@ -248,7 +267,7 @@ void GtkEpgWidget::create_channel_row(const Channel& const_channel,
 	guint number_columns = epg_span_hours * COLUMNS_PER_HOUR + 1;
 
 	if (!disable_epg)
-	{		
+	{
 		EpgEventList events = channel.epg_events.get_list();
 		for (EpgEventList::const_iterator i = events.begin(); i != events.end(); i++)
 		{
@@ -285,7 +304,7 @@ void GtkEpgWidget::create_channel_row(const Channel& const_channel,
 					{
 						guint empty_columns = start_column - total_number_columns;
 						Gtk::Button& button = attach_button(
-							empty_columns < 10 ? _("-") : _("Unknown program"),
+							empty_columns < 10 ? _("-") : _("Unknown program"), false,
 							total_number_columns + 1, start_column + 1, table_row, table_row + 1);
 						button.set_sensitive(false);
 						total_number_columns += empty_columns;
@@ -293,7 +312,7 @@ void GtkEpgWidget::create_channel_row(const Channel& const_channel,
 				
 					if (column_count > 0)
 					{
-						guint converted_start_time = convert_to_utc_time (epg_event.start_time);
+						guint converted_start_time = convert_to_utc_time(epg_event.start_time);
 
 						Glib::ustring text;
 						if (show_epg_time)
@@ -302,8 +321,10 @@ void GtkEpgWidget::create_channel_row(const Channel& const_channel,
 							text += get_local_time_text(converted_start_time + epg_event.duration, " - %H:%M</b>\n");
 						}
 						text += encode_xml(epg_event.get_title());
+
+						gboolean record = get_application().scheduled_recording_manager.is_recording(epg_event);
 						
-						Gtk::Button& button = attach_button(text, start_column + 1, end_column + 1, table_row, table_row + 1);
+						Gtk::Button& button = attach_button(text, record, start_column + 1, end_column + 1, table_row, table_row + 1);
 						button.signal_clicked().connect(
 							sigc::bind<EpgEvent>
 							(
@@ -311,6 +332,15 @@ void GtkEpgWidget::create_channel_row(const Channel& const_channel,
 								epg_event
 							)
 						);
+						button.signal_button_press_event().connect(
+							sigc::bind<EpgEvent>
+							(
+								sigc::mem_fun(*this, &GtkEpgWidget::on_button_program_press_event),
+								epg_event
+							),
+						    false
+						);
+
 
 						if (show_epg_tooltips)
 						{
@@ -331,38 +361,71 @@ void GtkEpgWidget::create_channel_row(const Channel& const_channel,
 	{
 		guint empty_columns = (number_columns-1) - total_number_columns;
 		Gtk::Button& button = attach_button(
-			empty_columns < 10 ? _("-") : _("Unknown program"),
+			empty_columns < 10 ? _("-") : _("Unknown program"), false,
 			total_number_columns + 1, number_columns, table_row, table_row + 1);
 		button.set_sensitive(false);
 	}
 }
 
-Gtk::ToggleButton& GtkEpgWidget::attach_toggle_button(const Glib::ustring& text, guint left_attach, guint right_attach, guint top_attach, guint bottom_attach, Gtk::AttachOptions attach_options)
+Gtk::ToggleButton& GtkEpgWidget::attach_toggle_button(const Glib::ustring& text, gboolean record, guint left_attach, guint right_attach, guint top_attach, guint bottom_attach, Gtk::AttachOptions attach_options)
 {
-	Gtk::ToggleButton* button = new Gtk::ToggleButton(text);
-	attach_widget(*button, left_attach, right_attach, top_attach, bottom_attach, attach_options);
+	Gtk::ToggleButton* button = new Gtk::ToggleButton();
 	button->set_alignment(0, 0.5);
-	Gtk::Label* label = dynamic_cast<Gtk::Label*>(button->get_child());
+
+	Gtk::HBox* hbox = Gtk::manage(new Gtk::HBox());
+
+	Gtk::Label* label = Gtk::manage(new Gtk::Label(text));
 	label->set_use_markup(true);
+	label->set_alignment(0, 0.5);
+	hbox->pack_start(*label, true, true);
+
+	if (record == true)
+	{
+		Gtk::Image* image = Gtk::manage(new Gtk::Image(Gtk::Stock::MEDIA_RECORD, Gtk::ICON_SIZE_BUTTON));
+		image->set_alignment(1, 0.5);
+		hbox->pack_end(*image, false, false);
+	}
+	
+	button->add(*hbox);	
+	button->show_all();
+
+	attach_widget(*button, left_attach, right_attach, top_attach, bottom_attach, attach_options);
+
 	return *button;
 }
 
-Gtk::Button& GtkEpgWidget::attach_button(const Glib::ustring& text, guint left_attach, guint right_attach, guint top_attach, guint bottom_attach, Gtk::AttachOptions attach_options)
+Gtk::Button& GtkEpgWidget::attach_button(const Glib::ustring& text, gboolean record, guint left_attach, guint right_attach, guint top_attach, guint bottom_attach, Gtk::AttachOptions attach_options)
 {
-	Gtk::Button* button = new Gtk::Button(text);
-	attach_widget(*button, left_attach, right_attach, top_attach, bottom_attach, attach_options);
+	Gtk::Button* button = new Gtk::Button();
 	button->set_alignment(0, 0.5);
-	Gtk::Label* label = dynamic_cast<Gtk::Label*>(button->get_child());
+
+	Gtk::HBox* hbox = Gtk::manage(new Gtk::HBox());
+
+	Gtk::Label* label = Gtk::manage(new Gtk::Label(text));
 	label->set_use_markup(true);
+	label->set_alignment(0, 0.5);
+	hbox->pack_start(*label, true, true);
+
+	if (record == true)
+	{
+		Gtk::Image* image = Gtk::manage(new Gtk::Image(Gtk::Stock::MEDIA_RECORD, Gtk::ICON_SIZE_BUTTON));
+		hbox->pack_end(*image, false, false);
+	}
+		
+	button->add(*hbox);	
+	button->show_all();
+
+	attach_widget(*button, left_attach, right_attach, top_attach, bottom_attach, attach_options);
+	
 	return *button;
 }
 
 Gtk::Label& GtkEpgWidget::attach_label(const Glib::ustring& text, guint left_attach, guint right_attach, guint top_attach, guint bottom_attach, Gtk::AttachOptions attach_options)
 {
 	Gtk::Label* label = new Gtk::Label(text.c_str());
-	attach_widget(*label, left_attach, right_attach, top_attach, bottom_attach, attach_options);
 	label->set_justify(Gtk::JUSTIFY_LEFT);
 	label->set_use_markup(true);
+	attach_widget(*label, left_attach, right_attach, top_attach, bottom_attach, attach_options);
 	return *label;
 }
 
@@ -372,7 +435,7 @@ void GtkEpgWidget::attach_widget(Gtk::Widget& widget, guint left_attach, guint r
 	widget.show();
 }
 
-void GtkEpgWidget::on_button_channel_name_clicked(guint channel_id)
+void GtkEpgWidget::on_button_channel_clicked(guint channel_id)
 {
 	TRY
 	get_application().set_display_channel_by_id(channel_id);
@@ -383,6 +446,42 @@ void GtkEpgWidget::on_button_channel_name_clicked(guint channel_id)
 	CATCH
 }
 
+bool GtkEpgWidget::on_button_channel_press_event(GdkEventButton* event, Channel& channel)
+{
+	if (event->type == GDK_BUTTON_PRESS && event->button == 3)
+	{
+		TRY
+		if (get_application().stream_manager.is_recording(channel))
+		{
+			get_application().stop_recording(channel);
+		}
+		else
+		{
+			get_application().start_recording(channel);
+		}
+		CATCH
+
+		TRY
+		update_table();
+		CATCH
+	}
+
+	return false;
+}
+
+bool GtkEpgWidget::on_button_program_press_event(GdkEventButton* event, EpgEvent& epg_event)
+{
+	if (event->type == GDK_BUTTON_PRESS && event->button == 3)
+	{
+		ScheduledRecordingDialog& scheduled_recording_dialog = ScheduledRecordingDialog::create(builder);
+		scheduled_recording_dialog.run(MainWindow::create(builder), epg_event);
+		scheduled_recording_dialog.hide();
+		get_application().update();
+	}
+
+	return false;
+}
+
 void GtkEpgWidget::on_button_program_clicked(EpgEvent& epg_event)
 {
 	TRY
@@ -413,6 +512,7 @@ void GtkEpgWidget::on_button_program_clicked(EpgEvent& epg_event)
 		ScheduledRecordingDialog& scheduled_recording_dialog = ScheduledRecordingDialog::create(builder);
 		scheduled_recording_dialog.run(MainWindow::create(builder), epg_event);
 		scheduled_recording_dialog.hide();
+		get_application().update();
 	}
 	
 	CATCH
