@@ -1,5 +1,5 @@
 	/*
- * Copyright (C) 2009 Michael Lamothe
+ * Copyright (C) 2010 Michael Lamothe
  *
  * This file is part of Me TV
  *
@@ -260,7 +260,7 @@ void ScanDialog::import_channels_conf(const Glib::ustring& channels_conf_path)
 
 					channel.name = channels_conf_line.get_name(0);
 					channel.sort_order = line_count;
-					channel.flags = CHANNEL_FLAG_DVB_T;
+					channel.type = FE_OFDM;
 			
 					channel.transponder.frontend_parameters.frequency						= channels_conf_line.get_int(1);
 					channel.transponder.frontend_parameters.inversion						= channels_conf_line.get_inversion(2);
@@ -283,7 +283,7 @@ void ScanDialog::import_channels_conf(const Glib::ustring& channels_conf_path)
 
 					channel.name = channels_conf_line.get_name(0);
 					channel.sort_order = line_count;
-					channel.flags = CHANNEL_FLAG_DVB_C;
+					channel.type = FE_QAM;
 			
 					channel.transponder.frontend_parameters.frequency			= channels_conf_line.get_int(1);
 					channel.transponder.frontend_parameters.inversion			= channels_conf_line.get_inversion(2);
@@ -302,7 +302,7 @@ void ScanDialog::import_channels_conf(const Glib::ustring& channels_conf_path)
 
 					channel.name = channels_conf_line.get_name(0);
 					channel.sort_order = line_count;
-					channel.flags = CHANNEL_FLAG_DVB_S;
+					channel.type = FE_QPSK;
 			
 					channel.transponder.frontend_parameters.frequency			= channels_conf_line.get_int(1)*1000;
 					channel.transponder.polarisation							= channels_conf_line.get_polarisation(2);
@@ -322,7 +322,7 @@ void ScanDialog::import_channels_conf(const Glib::ustring& channels_conf_path)
 
 					channel.name = channels_conf_line.get_name(0);
 					channel.sort_order = line_count;
-					channel.flags = CHANNEL_FLAG_ATSC;
+					channel.type = FE_ATSC;
 			
 					channel.transponder.frontend_parameters.frequency			= channels_conf_line.get_int(1);
 					channel.transponder.frontend_parameters.inversion			= INVERSION_AUTO;
@@ -380,16 +380,6 @@ void ScanDialog::on_button_scan_wizard_next_clicked()
 		builder->get_widget("file_chooser_button_scan", file_chooser_button);
 		initial_tuning_file = file_chooser_button->get_filename();
 
-		switch(frontend.get_frontend_info().type)
-		{
-			case FE_OFDM:
-			case FE_QAM:
-			case FE_QPSK:
-				break;
-			default:
-				throw Exception(_("Failed to scan: scanning is only supported for DVB-T, DVB-S and DVB-C devices"));
-		}
-
 		if (initial_tuning_file.empty())
 		{
 			throw Exception(_("No tuning file has been selected"));
@@ -408,7 +398,6 @@ void ScanDialog::on_button_scan_wizard_next_clicked()
 		scanner.signal_service.connect(sigc::mem_fun(*this, &ScanDialog::on_signal_service));
 		scanner.signal_progress.connect(sigc::mem_fun(*this, &ScanDialog::on_signal_progress));
 		scanner.signal_complete.connect(sigc::mem_fun(*this, &ScanDialog::on_signal_complete));
-		get_application().stop_stream_thread();
 		scan_thread->start();
 	}
 	else if (radio_button_import->get_active())
@@ -493,31 +482,9 @@ ChannelArray ScanDialog::get_channels()
 
 		channel.service_id			= row.get_value(columns.column_id);
 		channel.name				= row.get_value(columns.column_name);
+		channel.type				= frontend.get_frontend_info().type;
 		channel.transponder.frontend_parameters = row.get_value(columns.column_frontend_parameters);
 		channel.transponder.polarisation = row.get_value(columns.column_polarisation);
-
-		switch(frontend.get_frontend_info().type)
-		{
-			case FE_OFDM:
-				channel.flags = CHANNEL_FLAG_DVB_T;
-				break;
-				
-			case FE_QPSK:
-				channel.flags = CHANNEL_FLAG_DVB_S;
-				break;
-
-			case FE_QAM:
-				channel.flags = CHANNEL_FLAG_DVB_C;
-				break;
-				
-			case FE_ATSC:
-				channel.flags = CHANNEL_FLAG_ATSC;
-				break;
-
-			default:
-				throw Exception(_("Invalid frontend type"));
-				break;
-		}
 		
 		result.push_back(channel);
 		
